# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='messages',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0emessages.proto\x12\x08messages\"\xa4\x01\n\x08Response\x12(\n\x06status\x18\x01 \x01(\x0e\x32\x18.messages.ResponseStatus\x12\x34\n\x0eserviceMessage\x18\x02 \x01(\x0b\x32\x1c.messages.FromServiceMessage\x12\x38\n\x10\x63onnectorMessage\x18\x03 \x01(\x0b\x32\x1e.messages.FromConnectorMessage\"\xa3\x05\n\x14\x46romConnectorMessage\x12$\n\x08response\x18\x01 \x01(\x0b\x32\x12.messages.Response\x12:\n\x13gatewayTelemetryMsg\x18\x02 \x01(\x0b\x32\x1d.messages.GatewayTelemetryMsg\x12<\n\x14gatewayAttributesMsg\x18\x03 \x01(\x0b\x32\x1e.messages.GatewayAttributesMsg\x12\x32\n\x0fgatewayClaimMsg\x18\x04 \x01(\x0b\x32\x19.messages.GatewayClaimMsg\x12<\n\x14registerConnectorMsg\x18\x05 \x01(\x0b\x32\x1e.messages.RegisterConnectorMsg\x12@\n\x16unregisterConnectorMsg\x18\x06 \x01(\x0b\x32 .messages.UnregisterConnectorMsg\x12(\n\nconnectMsg\x18\x07 \x01(\x0b\x32\x14.messages.ConnectMsg\x12.\n\rdisconnectMsg\x18\x08 \x01(\x0b\x32\x17.messages.DisconnectMsg\x12>\n\x15gatewayRpcResponseMsg\x18\t \x01(\x0b\x32\x1f.messages.GatewayRpcResponseMsg\x12I\n\x1agatewayAttributeRequestMsg\x18\n \x01(\x0b\x32%.messages.GatewayAttributesRequestMsg\x12R\n\x1f\x63onnectorGetConnectedDevicesMsg\x18\x0b \x01(\x0b\x32).messages.ConnectorGetConnectedDevicesMsg\"\x9e\x04\n\x12\x46romServiceMessage\x12$\n\x08response\x18\x01 \x01(\x0b\x32\x12.messages.Response\x12\x46\n\x19\x63onnectorConfigurationMsg\x18\x02 \x01(\x0b\x32#.messages.ConnectorConfigurationMsg\x12^\n%gatewayAttributeUpdateNotificationMsg\x18\x03 \x01(\x0b\x32/.messages.GatewayAttributeUpdateNotificationMsg\x12J\n\x1bgatewayAttributeResponseMsg\x18\x04 \x01(\x0b\x32%.messages.GatewayAttributeResponseMsg\x12H\n\x1agatewayDeviceRpcRequestMsg\x18\x05 \x01(\x0b\x32$.messages.GatewayDeviceRpcRequestMsg\x12@\n\x16unregisterConnectorMsg\x18\x06 \x01(\x0b\x32 .messages.UnregisterConnectorMsg\x12\x62\n\'connectorGetConnectedDevicesResponseMsg\x18\x07 \x01(\x0b\x32\x31.messages.ConnectorGetConnectedDevicesResponseMsg\",\n\x14RegisterConnectorMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\".\n\x16UnregisterConnectorMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\"I\n\x19\x43onnectorConfigurationMsg\x12\x15\n\rconnectorName\x18\x01 \x01(\t\x12\x15\n\rconfiguration\x18\x02 \x01(\t\"7\n\x1f\x43onnectorGetConnectedDevicesMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\"b\n\'ConnectorGetConnectedDevicesResponseMsg\x12\x37\n\x10\x63onnectorDevices\x18\x01 \x03(\x0b\x32\x1d.messages.ConnectorDeviceInfo\"=\n\x13\x43onnectorDeviceInfo\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x12\n\ndeviceType\x18\x02 \x01(\t\"\x96\x01\n\rKeyValueProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x04type\x18\x02 \x01(\x0e\x32\x16.messages.KeyValueType\x12\x0e\n\x06\x62ool_v\x18\x03 \x01(\x08\x12\x0e\n\x06long_v\x18\x04 \x01(\x03\x12\x10\n\x08\x64ouble_v\x18\x05 \x01(\x01\x12\x10\n\x08string_v\x18\x06 \x01(\t\x12\x0e\n\x06json_v\x18\x07 \x01(\t\"<\n\tTsKvProto\x12\n\n\x02ts\x18\x01 \x01(\x03\x12#\n\x02kv\x18\x02 \x01(\x0b\x32\x17.messages.KeyValueProto\"@\n\rTsKvListProto\x12\n\n\x02ts\x18\x01 \x01(\x03\x12#\n\x02kv\x18\x02 \x03(\x0b\x32\x17.messages.KeyValueProto\"=\n\x10PostTelemetryMsg\x12)\n\x08tsKvList\x18\x01 \x03(\x0b\x32\x17.messages.TsKvListProto\"7\n\x10PostAttributeMsg\x12#\n\x02kv\x18\x01 \x03(\x0b\x32\x17.messages.KeyValueProto\"c\n\x1e\x41ttributeUpdateNotificationMsg\x12*\n\rsharedUpdated\x18\x01 \x03(\x0b\x32\x13.messages.TsKvProto\x12\x15\n\rsharedDeleted\x18\x02 \x03(\t\"N\n\x15ToDeviceRpcRequestMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x12\n\nmethodName\x18\x02 \x01(\t\x12\x0e\n\x06params\x18\x03 \x01(\t\"K\n\x16ToDeviceRpcResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x0f\n\x07payload\x18\x02 \x01(\t\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"N\n\x15ToServerRpcRequestMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x12\n\nmethodName\x18\x02 \x01(\t\x12\x0e\n\x06params\x18\x03 \x01(\t\"K\n\x16ToServerRpcResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x0f\n\x07payload\x18\x02 \x01(\t\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"4\n\x0b\x43laimDevice\x12\x11\n\tsecretKey\x18\x01 \x01(\t\x12\x12\n\ndurationMs\x18\x02 \x01(\x03\";\n\x11\x41ttributesRequest\x12\x12\n\nclientKeys\x18\x01 \x01(\t\x12\x12\n\nsharedKeys\x18\x02 \x01(\t\",\n\nRpcRequest\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\t\"#\n\rDisconnectMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\"4\n\nConnectMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x12\n\ndeviceType\x18\x02 \x01(\t\"K\n\x0cTelemetryMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\'\n\x03msg\x18\x03 \x01(\x0b\x32\x1a.messages.PostTelemetryMsg\"L\n\rAttributesMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\'\n\x03msg\x18\x02 \x01(\x0b\x32\x1a.messages.PostAttributeMsg\"Q\n\x0e\x43laimDeviceMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12+\n\x0c\x63laimRequest\x18\x02 \x01(\x0b\x32\x15.messages.ClaimDevice\":\n\x13GatewayTelemetryMsg\x12#\n\x03msg\x18\x01 \x03(\x0b\x32\x16.messages.TelemetryMsg\"8\n\x0fGatewayClaimMsg\x12%\n\x03msg\x18\x01 \x03(\x0b\x32\x18.messages.ClaimDeviceMsg\"<\n\x14GatewayAttributesMsg\x12$\n\x03msg\x18\x01 \x03(\x0b\x32\x17.messages.AttributesMsg\"E\n\x15GatewayRpcResponseMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t\"n\n\x1bGatewayAttributeResponseMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12;\n\x0bresponseMsg\x18\x02 \x01(\x0b\x32&.messages.GatewayAttributesResponseMsg\"~\n%GatewayAttributeUpdateNotificationMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x41\n\x0fnotificationMsg\x18\x02 \x01(\x0b\x32(.messages.AttributeUpdateNotificationMsg\"h\n\x1aGatewayDeviceRpcRequestMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x36\n\rrpcRequestMsg\x18\x02 \x01(\x0b\x32\x1f.messages.ToDeviceRpcRequestMsg\"[\n\x1bGatewayAttributesRequestMsg\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x12\n\ndeviceName\x18\x02 \x01(\t\x12\x0e\n\x06\x63lient\x18\x03 \x01(\x08\x12\x0c\n\x04keys\x18\x04 \x03(\t\"\xac\x01\n\x1cGatewayAttributesResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x34\n\x13\x63lientAttributeList\x18\x02 \x03(\x0b\x32\x17.messages.KeyValueProto\x12\x34\n\x13sharedAttributeList\x18\x03 \x03(\x0b\x32\x17.messages.KeyValueProto\x12\r\n\x05\x65rror\x18\x05 \x01(\t*F\n\x0eResponseStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\r\n\tNOT_FOUND\x10\x02\x12\x0b\n\x07\x46\x41ILURE\x10\x03*Q\n\x0cKeyValueType\x12\r\n\tBOOLEAN_V\x10\x00\x12\n\n\x06LONG_V\x10\x01\x12\x0c\n\x08\x44OUBLE_V\x10\x02\x12\x0c\n\x08STRING_V\x10\x03\x12\n\n\x06JSON_V\x10\x04\x32_\n\x15TBGatewayProtoService\x12\x46\n\x06stream\x12\x1e.messages.FromConnectorMessage\x1a\x1c.messages.FromServiceMessageb\x06proto3'
)

_RESPONSESTATUS = _descriptor.EnumDescriptor(
  name='ResponseStatus',
  full_name='messages.ResponseStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_FOUND', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3978,
  serialized_end=4048,
)
_sym_db.RegisterEnumDescriptor(_RESPONSESTATUS)

ResponseStatus = enum_type_wrapper.EnumTypeWrapper(_RESPONSESTATUS)
_KEYVALUETYPE = _descriptor.EnumDescriptor(
  name='KeyValueType',
  full_name='messages.KeyValueType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BOOLEAN_V', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LONG_V', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE_V', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING_V', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JSON_V', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4050,
  serialized_end=4131,
)
_sym_db.RegisterEnumDescriptor(_KEYVALUETYPE)

KeyValueType = enum_type_wrapper.EnumTypeWrapper(_KEYVALUETYPE)
UNKNOWN = 0
SUCCESS = 1
NOT_FOUND = 2
FAILURE = 3
BOOLEAN_V = 0
LONG_V = 1
DOUBLE_V = 2
STRING_V = 3
JSON_V = 4



_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='messages.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='messages.Response.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serviceMessage', full_name='messages.Response.serviceMessage', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorMessage', full_name='messages.Response.connectorMessage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=193,
)


_FROMCONNECTORMESSAGE = _descriptor.Descriptor(
  name='FromConnectorMessage',
  full_name='messages.FromConnectorMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='messages.FromConnectorMessage.response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayTelemetryMsg', full_name='messages.FromConnectorMessage.gatewayTelemetryMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributesMsg', full_name='messages.FromConnectorMessage.gatewayAttributesMsg', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayClaimMsg', full_name='messages.FromConnectorMessage.gatewayClaimMsg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='registerConnectorMsg', full_name='messages.FromConnectorMessage.registerConnectorMsg', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unregisterConnectorMsg', full_name='messages.FromConnectorMessage.unregisterConnectorMsg', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectMsg', full_name='messages.FromConnectorMessage.connectMsg', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disconnectMsg', full_name='messages.FromConnectorMessage.disconnectMsg', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayRpcResponseMsg', full_name='messages.FromConnectorMessage.gatewayRpcResponseMsg', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeRequestMsg', full_name='messages.FromConnectorMessage.gatewayAttributeRequestMsg', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorGetConnectedDevicesMsg', full_name='messages.FromConnectorMessage.connectorGetConnectedDevicesMsg', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=871,
)


_FROMSERVICEMESSAGE = _descriptor.Descriptor(
  name='FromServiceMessage',
  full_name='messages.FromServiceMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='messages.FromServiceMessage.response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorConfigurationMsg', full_name='messages.FromServiceMessage.connectorConfigurationMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeUpdateNotificationMsg', full_name='messages.FromServiceMessage.gatewayAttributeUpdateNotificationMsg', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeResponseMsg', full_name='messages.FromServiceMessage.gatewayAttributeResponseMsg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayDeviceRpcRequestMsg', full_name='messages.FromServiceMessage.gatewayDeviceRpcRequestMsg', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unregisterConnectorMsg', full_name='messages.FromServiceMessage.unregisterConnectorMsg', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorGetConnectedDevicesResponseMsg', full_name='messages.FromServiceMessage.connectorGetConnectedDevicesResponseMsg', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=874,
  serialized_end=1416,
)


_REGISTERCONNECTORMSG = _descriptor.Descriptor(
  name='RegisterConnectorMsg',
  full_name='messages.RegisterConnectorMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.RegisterConnectorMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1418,
  serialized_end=1462,
)


_UNREGISTERCONNECTORMSG = _descriptor.Descriptor(
  name='UnregisterConnectorMsg',
  full_name='messages.UnregisterConnectorMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.UnregisterConnectorMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1464,
  serialized_end=1510,
)


_CONNECTORCONFIGURATIONMSG = _descriptor.Descriptor(
  name='ConnectorConfigurationMsg',
  full_name='messages.ConnectorConfigurationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorName', full_name='messages.ConnectorConfigurationMsg.connectorName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='configuration', full_name='messages.ConnectorConfigurationMsg.configuration', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1512,
  serialized_end=1585,
)


_CONNECTORGETCONNECTEDDEVICESMSG = _descriptor.Descriptor(
  name='ConnectorGetConnectedDevicesMsg',
  full_name='messages.ConnectorGetConnectedDevicesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.ConnectorGetConnectedDevicesMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1587,
  serialized_end=1642,
)


_CONNECTORGETCONNECTEDDEVICESRESPONSEMSG = _descriptor.Descriptor(
  name='ConnectorGetConnectedDevicesResponseMsg',
  full_name='messages.ConnectorGetConnectedDevicesResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorDevices', full_name='messages.ConnectorGetConnectedDevicesResponseMsg.connectorDevices', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1644,
  serialized_end=1742,
)


_CONNECTORDEVICEINFO = _descriptor.Descriptor(
  name='ConnectorDeviceInfo',
  full_name='messages.ConnectorDeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ConnectorDeviceInfo.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceType', full_name='messages.ConnectorDeviceInfo.deviceType', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1744,
  serialized_end=1805,
)


_KEYVALUEPROTO = _descriptor.Descriptor(
  name='KeyValueProto',
  full_name='messages.KeyValueProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='messages.KeyValueProto.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='messages.KeyValueProto.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_v', full_name='messages.KeyValueProto.bool_v', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='long_v', full_name='messages.KeyValueProto.long_v', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_v', full_name='messages.KeyValueProto.double_v', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_v', full_name='messages.KeyValueProto.string_v', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='json_v', full_name='messages.KeyValueProto.json_v', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1808,
  serialized_end=1958,
)


_TSKVPROTO = _descriptor.Descriptor(
  name='TsKvProto',
  full_name='messages.TsKvProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ts', full_name='messages.TsKvProto.ts', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.TsKvProto.kv', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1960,
  serialized_end=2020,
)


_TSKVLISTPROTO = _descriptor.Descriptor(
  name='TsKvListProto',
  full_name='messages.TsKvListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ts', full_name='messages.TsKvListProto.ts', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.TsKvListProto.kv', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2022,
  serialized_end=2086,
)


_POSTTELEMETRYMSG = _descriptor.Descriptor(
  name='PostTelemetryMsg',
  full_name='messages.PostTelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tsKvList', full_name='messages.PostTelemetryMsg.tsKvList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2088,
  serialized_end=2149,
)


_POSTATTRIBUTEMSG = _descriptor.Descriptor(
  name='PostAttributeMsg',
  full_name='messages.PostAttributeMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.PostAttributeMsg.kv', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2151,
  serialized_end=2206,
)


_ATTRIBUTEUPDATENOTIFICATIONMSG = _descriptor.Descriptor(
  name='AttributeUpdateNotificationMsg',
  full_name='messages.AttributeUpdateNotificationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharedUpdated', full_name='messages.AttributeUpdateNotificationMsg.sharedUpdated', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedDeleted', full_name='messages.AttributeUpdateNotificationMsg.sharedDeleted', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2208,
  serialized_end=2307,
)


_TODEVICERPCREQUESTMSG = _descriptor.Descriptor(
  name='ToDeviceRpcRequestMsg',
  full_name='messages.ToDeviceRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToDeviceRpcRequestMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='methodName', full_name='messages.ToDeviceRpcRequestMsg.methodName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.ToDeviceRpcRequestMsg.params', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2309,
  serialized_end=2387,
)


_TODEVICERPCRESPONSEMSG = _descriptor.Descriptor(
  name='ToDeviceRpcResponseMsg',
  full_name='messages.ToDeviceRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToDeviceRpcResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='messages.ToDeviceRpcResponseMsg.payload', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.ToDeviceRpcResponseMsg.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2389,
  serialized_end=2464,
)


_TOSERVERRPCREQUESTMSG = _descriptor.Descriptor(
  name='ToServerRpcRequestMsg',
  full_name='messages.ToServerRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToServerRpcRequestMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='methodName', full_name='messages.ToServerRpcRequestMsg.methodName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.ToServerRpcRequestMsg.params', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2466,
  serialized_end=2544,
)


_TOSERVERRPCRESPONSEMSG = _descriptor.Descriptor(
  name='ToServerRpcResponseMsg',
  full_name='messages.ToServerRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToServerRpcResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='messages.ToServerRpcResponseMsg.payload', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.ToServerRpcResponseMsg.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2546,
  serialized_end=2621,
)


_CLAIMDEVICE = _descriptor.Descriptor(
  name='ClaimDevice',
  full_name='messages.ClaimDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='secretKey', full_name='messages.ClaimDevice.secretKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='durationMs', full_name='messages.ClaimDevice.durationMs', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2623,
  serialized_end=2675,
)


_ATTRIBUTESREQUEST = _descriptor.Descriptor(
  name='AttributesRequest',
  full_name='messages.AttributesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientKeys', full_name='messages.AttributesRequest.clientKeys', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedKeys', full_name='messages.AttributesRequest.sharedKeys', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2677,
  serialized_end=2736,
)


_RPCREQUEST = _descriptor.Descriptor(
  name='RpcRequest',
  full_name='messages.RpcRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='messages.RpcRequest.method', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.RpcRequest.params', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2738,
  serialized_end=2782,
)


_DISCONNECTMSG = _descriptor.Descriptor(
  name='DisconnectMsg',
  full_name='messages.DisconnectMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.DisconnectMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2784,
  serialized_end=2819,
)


_CONNECTMSG = _descriptor.Descriptor(
  name='ConnectMsg',
  full_name='messages.ConnectMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ConnectMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceType', full_name='messages.ConnectMsg.deviceType', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2821,
  serialized_end=2873,
)


_TELEMETRYMSG = _descriptor.Descriptor(
  name='TelemetryMsg',
  full_name='messages.TelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.TelemetryMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.TelemetryMsg.msg', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2875,
  serialized_end=2950,
)


_ATTRIBUTESMSG = _descriptor.Descriptor(
  name='AttributesMsg',
  full_name='messages.AttributesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.AttributesMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.AttributesMsg.msg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2952,
  serialized_end=3028,
)


_CLAIMDEVICEMSG = _descriptor.Descriptor(
  name='ClaimDeviceMsg',
  full_name='messages.ClaimDeviceMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ClaimDeviceMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='claimRequest', full_name='messages.ClaimDeviceMsg.claimRequest', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3030,
  serialized_end=3111,
)


_GATEWAYTELEMETRYMSG = _descriptor.Descriptor(
  name='GatewayTelemetryMsg',
  full_name='messages.GatewayTelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayTelemetryMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3113,
  serialized_end=3171,
)


_GATEWAYCLAIMMSG = _descriptor.Descriptor(
  name='GatewayClaimMsg',
  full_name='messages.GatewayClaimMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayClaimMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3173,
  serialized_end=3229,
)


_GATEWAYATTRIBUTESMSG = _descriptor.Descriptor(
  name='GatewayAttributesMsg',
  full_name='messages.GatewayAttributesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayAttributesMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3231,
  serialized_end=3291,
)


_GATEWAYRPCRESPONSEMSG = _descriptor.Descriptor(
  name='GatewayRpcResponseMsg',
  full_name='messages.GatewayRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayRpcResponseMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='messages.GatewayRpcResponseMsg.id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='messages.GatewayRpcResponseMsg.data', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3293,
  serialized_end=3362,
)


_GATEWAYATTRIBUTERESPONSEMSG = _descriptor.Descriptor(
  name='GatewayAttributeResponseMsg',
  full_name='messages.GatewayAttributeResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributeResponseMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='responseMsg', full_name='messages.GatewayAttributeResponseMsg.responseMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3364,
  serialized_end=3474,
)


_GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG = _descriptor.Descriptor(
  name='GatewayAttributeUpdateNotificationMsg',
  full_name='messages.GatewayAttributeUpdateNotificationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributeUpdateNotificationMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='notificationMsg', full_name='messages.GatewayAttributeUpdateNotificationMsg.notificationMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3476,
  serialized_end=3602,
)


_GATEWAYDEVICERPCREQUESTMSG = _descriptor.Descriptor(
  name='GatewayDeviceRpcRequestMsg',
  full_name='messages.GatewayDeviceRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayDeviceRpcRequestMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rpcRequestMsg', full_name='messages.GatewayDeviceRpcRequestMsg.rpcRequestMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3604,
  serialized_end=3708,
)


_GATEWAYATTRIBUTESREQUESTMSG = _descriptor.Descriptor(
  name='GatewayAttributesRequestMsg',
  full_name='messages.GatewayAttributesRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='messages.GatewayAttributesRequestMsg.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributesRequestMsg.deviceName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client', full_name='messages.GatewayAttributesRequestMsg.client', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='keys', full_name='messages.GatewayAttributesRequestMsg.keys', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3710,
  serialized_end=3801,
)


_GATEWAYATTRIBUTESRESPONSEMSG = _descriptor.Descriptor(
  name='GatewayAttributesResponseMsg',
  full_name='messages.GatewayAttributesResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.GatewayAttributesResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clientAttributeList', full_name='messages.GatewayAttributesResponseMsg.clientAttributeList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedAttributeList', full_name='messages.GatewayAttributesResponseMsg.sharedAttributeList', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.GatewayAttributesResponseMsg.error', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3804,
  serialized_end=3976,
)

_RESPONSE.fields_by_name['status'].enum_type = _RESPONSESTATUS
_RESPONSE.fields_by_name['serviceMessage'].message_type = _FROMSERVICEMESSAGE
_RESPONSE.fields_by_name['connectorMessage'].message_type = _FROMCONNECTORMESSAGE
_FROMCONNECTORMESSAGE.fields_by_name['response'].message_type = _RESPONSE
_FROMCONNECTORMESSAGE.fields_by_name['gatewayTelemetryMsg'].message_type = _GATEWAYTELEMETRYMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayAttributesMsg'].message_type = _GATEWAYATTRIBUTESMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayClaimMsg'].message_type = _GATEWAYCLAIMMSG
_FROMCONNECTORMESSAGE.fields_by_name['registerConnectorMsg'].message_type = _REGISTERCONNECTORMSG
_FROMCONNECTORMESSAGE.fields_by_name['unregisterConnectorMsg'].message_type = _UNREGISTERCONNECTORMSG
_FROMCONNECTORMESSAGE.fields_by_name['connectMsg'].message_type = _CONNECTMSG
_FROMCONNECTORMESSAGE.fields_by_name['disconnectMsg'].message_type = _DISCONNECTMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayRpcResponseMsg'].message_type = _GATEWAYRPCRESPONSEMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayAttributeRequestMsg'].message_type = _GATEWAYATTRIBUTESREQUESTMSG
_FROMCONNECTORMESSAGE.fields_by_name['connectorGetConnectedDevicesMsg'].message_type = _CONNECTORGETCONNECTEDDEVICESMSG
_FROMSERVICEMESSAGE.fields_by_name['response'].message_type = _RESPONSE
_FROMSERVICEMESSAGE.fields_by_name['connectorConfigurationMsg'].message_type = _CONNECTORCONFIGURATIONMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayAttributeUpdateNotificationMsg'].message_type = _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayAttributeResponseMsg'].message_type = _GATEWAYATTRIBUTERESPONSEMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayDeviceRpcRequestMsg'].message_type = _GATEWAYDEVICERPCREQUESTMSG
_FROMSERVICEMESSAGE.fields_by_name['unregisterConnectorMsg'].message_type = _UNREGISTERCONNECTORMSG
_FROMSERVICEMESSAGE.fields_by_name['connectorGetConnectedDevicesResponseMsg'].message_type = _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG
_CONNECTORGETCONNECTEDDEVICESRESPONSEMSG.fields_by_name['connectorDevices'].message_type = _CONNECTORDEVICEINFO
_KEYVALUEPROTO.fields_by_name['type'].enum_type = _KEYVALUETYPE
_TSKVPROTO.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_TSKVLISTPROTO.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_POSTTELEMETRYMSG.fields_by_name['tsKvList'].message_type = _TSKVLISTPROTO
_POSTATTRIBUTEMSG.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_ATTRIBUTEUPDATENOTIFICATIONMSG.fields_by_name['sharedUpdated'].message_type = _TSKVPROTO
_TELEMETRYMSG.fields_by_name['msg'].message_type = _POSTTELEMETRYMSG
_ATTRIBUTESMSG.fields_by_name['msg'].message_type = _POSTATTRIBUTEMSG
_CLAIMDEVICEMSG.fields_by_name['claimRequest'].message_type = _CLAIMDEVICE
_GATEWAYTELEMETRYMSG.fields_by_name['msg'].message_type = _TELEMETRYMSG
_GATEWAYCLAIMMSG.fields_by_name['msg'].message_type = _CLAIMDEVICEMSG
_GATEWAYATTRIBUTESMSG.fields_by_name['msg'].message_type = _ATTRIBUTESMSG
_GATEWAYATTRIBUTERESPONSEMSG.fields_by_name['responseMsg'].message_type = _GATEWAYATTRIBUTESRESPONSEMSG
_GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG.fields_by_name['notificationMsg'].message_type = _ATTRIBUTEUPDATENOTIFICATIONMSG
_GATEWAYDEVICERPCREQUESTMSG.fields_by_name['rpcRequestMsg'].message_type = _TODEVICERPCREQUESTMSG
_GATEWAYATTRIBUTESRESPONSEMSG.fields_by_name['clientAttributeList'].message_type = _KEYVALUEPROTO
_GATEWAYATTRIBUTESRESPONSEMSG.fields_by_name['sharedAttributeList'].message_type = _KEYVALUEPROTO
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['FromConnectorMessage'] = _FROMCONNECTORMESSAGE
DESCRIPTOR.message_types_by_name['FromServiceMessage'] = _FROMSERVICEMESSAGE
DESCRIPTOR.message_types_by_name['RegisterConnectorMsg'] = _REGISTERCONNECTORMSG
DESCRIPTOR.message_types_by_name['UnregisterConnectorMsg'] = _UNREGISTERCONNECTORMSG
DESCRIPTOR.message_types_by_name['ConnectorConfigurationMsg'] = _CONNECTORCONFIGURATIONMSG
DESCRIPTOR.message_types_by_name['ConnectorGetConnectedDevicesMsg'] = _CONNECTORGETCONNECTEDDEVICESMSG
DESCRIPTOR.message_types_by_name['ConnectorGetConnectedDevicesResponseMsg'] = _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG
DESCRIPTOR.message_types_by_name['ConnectorDeviceInfo'] = _CONNECTORDEVICEINFO
DESCRIPTOR.message_types_by_name['KeyValueProto'] = _KEYVALUEPROTO
DESCRIPTOR.message_types_by_name['TsKvProto'] = _TSKVPROTO
DESCRIPTOR.message_types_by_name['TsKvListProto'] = _TSKVLISTPROTO
DESCRIPTOR.message_types_by_name['PostTelemetryMsg'] = _POSTTELEMETRYMSG
DESCRIPTOR.message_types_by_name['PostAttributeMsg'] = _POSTATTRIBUTEMSG
DESCRIPTOR.message_types_by_name['AttributeUpdateNotificationMsg'] = _ATTRIBUTEUPDATENOTIFICATIONMSG
DESCRIPTOR.message_types_by_name['ToDeviceRpcRequestMsg'] = _TODEVICERPCREQUESTMSG
DESCRIPTOR.message_types_by_name['ToDeviceRpcResponseMsg'] = _TODEVICERPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['ToServerRpcRequestMsg'] = _TOSERVERRPCREQUESTMSG
DESCRIPTOR.message_types_by_name['ToServerRpcResponseMsg'] = _TOSERVERRPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['ClaimDevice'] = _CLAIMDEVICE
DESCRIPTOR.message_types_by_name['AttributesRequest'] = _ATTRIBUTESREQUEST
DESCRIPTOR.message_types_by_name['RpcRequest'] = _RPCREQUEST
DESCRIPTOR.message_types_by_name['DisconnectMsg'] = _DISCONNECTMSG
DESCRIPTOR.message_types_by_name['ConnectMsg'] = _CONNECTMSG
DESCRIPTOR.message_types_by_name['TelemetryMsg'] = _TELEMETRYMSG
DESCRIPTOR.message_types_by_name['AttributesMsg'] = _ATTRIBUTESMSG
DESCRIPTOR.message_types_by_name['ClaimDeviceMsg'] = _CLAIMDEVICEMSG
DESCRIPTOR.message_types_by_name['GatewayTelemetryMsg'] = _GATEWAYTELEMETRYMSG
DESCRIPTOR.message_types_by_name['GatewayClaimMsg'] = _GATEWAYCLAIMMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesMsg'] = _GATEWAYATTRIBUTESMSG
DESCRIPTOR.message_types_by_name['GatewayRpcResponseMsg'] = _GATEWAYRPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['GatewayAttributeResponseMsg'] = _GATEWAYATTRIBUTERESPONSEMSG
DESCRIPTOR.message_types_by_name['GatewayAttributeUpdateNotificationMsg'] = _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG
DESCRIPTOR.message_types_by_name['GatewayDeviceRpcRequestMsg'] = _GATEWAYDEVICERPCREQUESTMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesRequestMsg'] = _GATEWAYATTRIBUTESREQUESTMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesResponseMsg'] = _GATEWAYATTRIBUTESRESPONSEMSG
DESCRIPTOR.enum_types_by_name['ResponseStatus'] = _RESPONSESTATUS
DESCRIPTOR.enum_types_by_name['KeyValueType'] = _KEYVALUETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.Response)
  })
_sym_db.RegisterMessage(Response)

FromConnectorMessage = _reflection.GeneratedProtocolMessageType('FromConnectorMessage', (_message.Message,), {
  'DESCRIPTOR' : _FROMCONNECTORMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.FromConnectorMessage)
  })
_sym_db.RegisterMessage(FromConnectorMessage)

FromServiceMessage = _reflection.GeneratedProtocolMessageType('FromServiceMessage', (_message.Message,), {
  'DESCRIPTOR' : _FROMSERVICEMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.FromServiceMessage)
  })
_sym_db.RegisterMessage(FromServiceMessage)

RegisterConnectorMsg = _reflection.GeneratedProtocolMessageType('RegisterConnectorMsg', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERCONNECTORMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.RegisterConnectorMsg)
  })
_sym_db.RegisterMessage(RegisterConnectorMsg)

UnregisterConnectorMsg = _reflection.GeneratedProtocolMessageType('UnregisterConnectorMsg', (_message.Message,), {
  'DESCRIPTOR' : _UNREGISTERCONNECTORMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.UnregisterConnectorMsg)
  })
_sym_db.RegisterMessage(UnregisterConnectorMsg)

ConnectorConfigurationMsg = _reflection.GeneratedProtocolMessageType('ConnectorConfigurationMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORCONFIGURATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorConfigurationMsg)
  })
_sym_db.RegisterMessage(ConnectorConfigurationMsg)

ConnectorGetConnectedDevicesMsg = _reflection.GeneratedProtocolMessageType('ConnectorGetConnectedDevicesMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORGETCONNECTEDDEVICESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorGetConnectedDevicesMsg)
  })
_sym_db.RegisterMessage(ConnectorGetConnectedDevicesMsg)

ConnectorGetConnectedDevicesResponseMsg = _reflection.GeneratedProtocolMessageType('ConnectorGetConnectedDevicesResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorGetConnectedDevicesResponseMsg)
  })
_sym_db.RegisterMessage(ConnectorGetConnectedDevicesResponseMsg)

ConnectorDeviceInfo = _reflection.GeneratedProtocolMessageType('ConnectorDeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORDEVICEINFO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorDeviceInfo)
  })
_sym_db.RegisterMessage(ConnectorDeviceInfo)

KeyValueProto = _reflection.GeneratedProtocolMessageType('KeyValueProto', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.KeyValueProto)
  })
_sym_db.RegisterMessage(KeyValueProto)

TsKvProto = _reflection.GeneratedProtocolMessageType('TsKvProto', (_message.Message,), {
  'DESCRIPTOR' : _TSKVPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TsKvProto)
  })
_sym_db.RegisterMessage(TsKvProto)

TsKvListProto = _reflection.GeneratedProtocolMessageType('TsKvListProto', (_message.Message,), {
  'DESCRIPTOR' : _TSKVLISTPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TsKvListProto)
  })
_sym_db.RegisterMessage(TsKvListProto)

PostTelemetryMsg = _reflection.GeneratedProtocolMessageType('PostTelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _POSTTELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.PostTelemetryMsg)
  })
_sym_db.RegisterMessage(PostTelemetryMsg)

PostAttributeMsg = _reflection.GeneratedProtocolMessageType('PostAttributeMsg', (_message.Message,), {
  'DESCRIPTOR' : _POSTATTRIBUTEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.PostAttributeMsg)
  })
_sym_db.RegisterMessage(PostAttributeMsg)

AttributeUpdateNotificationMsg = _reflection.GeneratedProtocolMessageType('AttributeUpdateNotificationMsg', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEUPDATENOTIFICATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributeUpdateNotificationMsg)
  })
_sym_db.RegisterMessage(AttributeUpdateNotificationMsg)

ToDeviceRpcRequestMsg = _reflection.GeneratedProtocolMessageType('ToDeviceRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _TODEVICERPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToDeviceRpcRequestMsg)
  })
_sym_db.RegisterMessage(ToDeviceRpcRequestMsg)

ToDeviceRpcResponseMsg = _reflection.GeneratedProtocolMessageType('ToDeviceRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _TODEVICERPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToDeviceRpcResponseMsg)
  })
_sym_db.RegisterMessage(ToDeviceRpcResponseMsg)

ToServerRpcRequestMsg = _reflection.GeneratedProtocolMessageType('ToServerRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _TOSERVERRPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToServerRpcRequestMsg)
  })
_sym_db.RegisterMessage(ToServerRpcRequestMsg)

ToServerRpcResponseMsg = _reflection.GeneratedProtocolMessageType('ToServerRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _TOSERVERRPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToServerRpcResponseMsg)
  })
_sym_db.RegisterMessage(ToServerRpcResponseMsg)

ClaimDevice = _reflection.GeneratedProtocolMessageType('ClaimDevice', (_message.Message,), {
  'DESCRIPTOR' : _CLAIMDEVICE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ClaimDevice)
  })
_sym_db.RegisterMessage(ClaimDevice)

AttributesRequest = _reflection.GeneratedProtocolMessageType('AttributesRequest', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTESREQUEST,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributesRequest)
  })
_sym_db.RegisterMessage(AttributesRequest)

RpcRequest = _reflection.GeneratedProtocolMessageType('RpcRequest', (_message.Message,), {
  'DESCRIPTOR' : _RPCREQUEST,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.RpcRequest)
  })
_sym_db.RegisterMessage(RpcRequest)

DisconnectMsg = _reflection.GeneratedProtocolMessageType('DisconnectMsg', (_message.Message,), {
  'DESCRIPTOR' : _DISCONNECTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.DisconnectMsg)
  })
_sym_db.RegisterMessage(DisconnectMsg)

ConnectMsg = _reflection.GeneratedProtocolMessageType('ConnectMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectMsg)
  })
_sym_db.RegisterMessage(ConnectMsg)

TelemetryMsg = _reflection.GeneratedProtocolMessageType('TelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _TELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TelemetryMsg)
  })
_sym_db.RegisterMessage(TelemetryMsg)

AttributesMsg = _reflection.GeneratedProtocolMessageType('AttributesMsg', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributesMsg)
  })
_sym_db.RegisterMessage(AttributesMsg)

ClaimDeviceMsg = _reflection.GeneratedProtocolMessageType('ClaimDeviceMsg', (_message.Message,), {
  'DESCRIPTOR' : _CLAIMDEVICEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ClaimDeviceMsg)
  })
_sym_db.RegisterMessage(ClaimDeviceMsg)

GatewayTelemetryMsg = _reflection.GeneratedProtocolMessageType('GatewayTelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYTELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayTelemetryMsg)
  })
_sym_db.RegisterMessage(GatewayTelemetryMsg)

GatewayClaimMsg = _reflection.GeneratedProtocolMessageType('GatewayClaimMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYCLAIMMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayClaimMsg)
  })
_sym_db.RegisterMessage(GatewayClaimMsg)

GatewayAttributesMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesMsg)

GatewayRpcResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYRPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayRpcResponseMsg)
  })
_sym_db.RegisterMessage(GatewayRpcResponseMsg)

GatewayAttributeResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributeResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTERESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributeResponseMsg)
  })
_sym_db.RegisterMessage(GatewayAttributeResponseMsg)

GatewayAttributeUpdateNotificationMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributeUpdateNotificationMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributeUpdateNotificationMsg)
  })
_sym_db.RegisterMessage(GatewayAttributeUpdateNotificationMsg)

GatewayDeviceRpcRequestMsg = _reflection.GeneratedProtocolMessageType('GatewayDeviceRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYDEVICERPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayDeviceRpcRequestMsg)
  })
_sym_db.RegisterMessage(GatewayDeviceRpcRequestMsg)

GatewayAttributesRequestMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesRequestMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesRequestMsg)

GatewayAttributesResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesResponseMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesResponseMsg)



_TBGATEWAYPROTOSERVICE = _descriptor.ServiceDescriptor(
  name='TBGatewayProtoService',
  full_name='messages.TBGatewayProtoService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=4133,
  serialized_end=4228,
  methods=[
  _descriptor.MethodDescriptor(
    name='stream',
    full_name='messages.TBGatewayProtoService.stream',
    index=0,
    containing_service=None,
    input_type=_FROMCONNECTORMESSAGE,
    output_type=_FROMSERVICEMESSAGE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TBGATEWAYPROTOSERVICE)

DESCRIPTOR.services_by_name['TBGatewayProtoService'] = _TBGATEWAYPROTOSERVICE

# @@protoc_insertion_point(module_scope)



#     Copyright 2022. ThingsBoard
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.

import asyncio
from random import choice
from re import search
from socket import gethostbyname
from string import ascii_lowercase
from threading import Thread
from time import sleep, time

from thingsboard_gateway.connectors.connector import Connector
from thingsboard_gateway.tb_utility.tb_loader import TBModuleLoader
from thingsboard_gateway.tb_utility.tb_utility import TBUtility
from thingsboard_gateway.tb_utility.tb_logger import init_logger

# Try import Pymodbus library or install it and import
installation_required = False

try:
    from puresnmp import __version__ as pymodbus_version

    if int(pymodbus_version.split('.')[0]) < 2:
        installation_required = True
except ImportError:
    installation_required = True

if installation_required:
    print("Modbus library not found - installing...")
    TBUtility.install_package("puresnmp", ">=2.0.0")

from puresnmp import Client, credentials, PyWrapper
from puresnmp.exc import Timeout as SNMPTimeoutException


class SNMPConnector(Connector, Thread):
    def __init__(self, gateway, config, connector_type):
        super().__init__()
        self.daemon = True
        self.__gateway = gateway
        self._connected = False
        self.__stopped = False
        self._connector_type = connector_type
        self.__config = config
        self.setName(config.get("name", 'SNMP Connector ' + ''.join(choice(ascii_lowercase) for _ in range(5))))
        self._log = init_logger(self.__gateway, self.name, self.__config.get('logLevel', 'INFO'))
        self.__devices = self.__config["devices"]
        self.statistics = {'MessagesReceived': 0,
                           'MessagesSent': 0}
        self._default_converters = {
            "uplink": "SNMPUplinkConverter",
            "downlink": "SNMPDownlinkConverter"
        }
        self.__methods = ["get", "multiget", "getnext", "walk", "multiwalk", "set", "multiset",
                          "bulkget", "bulkwalk", "table", "bulktable"]
        self.__datatypes = ('attributes', 'telemetry')

        self.__loop = asyncio.new_event_loop()

    def open(self):
        self.__stopped = False
        self.__fill_converters()
        self.start()

    def run(self):
        self._connected = True
        try:
            self.__loop.run_until_complete(self._run())
        except Exception as e:
            self._log.exception(e)

    async def _run(self):
        while not self.__stopped:
            current_time = time() * 1000
            for device in self.__devices:
                try:
                    if device.get("previous_poll_time", 0) + device.get("pollPeriod", 10000) < current_time:
                        await self.__process_data(device)
                        device["previous_poll_time"] = current_time
                except Exception as e:
                    self._log.exception(e)
            if self.__stopped:
                break
            else:
                sleep(.2)

    def close(self):
        self.__stopped = True
        self._connected = False

    def get_name(self):
        return self.name

    def get_type(self):
        return self._connector_type

    def is_connected(self):
        return self._connected

    def get_config(self):
        return self.__config

    def collect_statistic_and_send(self, connector_name, data):
        self.statistics["MessagesReceived"] = self.statistics["MessagesReceived"] + 1
        self.__gateway.send_to_storage(connector_name, data)
        self.statistics["MessagesSent"] = self.statistics["MessagesSent"] + 1

    async def __process_data(self, device):
        common_parameters = self.__get_common_parameters(device)
        converted_data = {}
        for datatype in self.__datatypes:
            for datatype_config in device[datatype]:
                try:
                    response = None
                    method = datatype_config.get("method")
                    if method is None:
                        self._log.error("Method not found in configuration: %r", datatype_config)
                        continue
                    else:
                        method = method.lower()
                    if method not in self.__methods:
                        self._log.error("Unknown method: %s, configuration is: %r", method, datatype_config)
                    response = await self.__process_methods(method, common_parameters, datatype_config)
                    converted_data.update(**device["uplink_converter"].convert((datatype, datatype_config), response))
                except SNMPTimeoutException:
                    self._log.error("Timeout exception on connection to device \"%s\" with ip: \"%s\"", device["deviceName"],
                              device["ip"])
                    return
                except Exception as e:
                    self._log.exception(e)

        if isinstance(converted_data, dict) and (converted_data.get("attributes") or converted_data.get("telemetry")):
            self.collect_statistic_and_send(self.get_name(), converted_data)

    async def __process_methods(self, method, common_parameters, datatype_config):
        client = Client(ip=common_parameters['ip'],
                        port=common_parameters['port'],
                        credentials=credentials.V1(common_parameters['community']))
        client.configure(timeout=common_parameters['timeout'])
        client = PyWrapper(client)

        response = None

        if method == "get":
            oid = datatype_config["oid"]
            response = await client.get(oid=oid)
        elif method == "multiget":
            oids = datatype_config["oid"]
            oids = oids if isinstance(oids, list) else list(oids)
            response = await client.multiget(oids=oids)
        elif method == "getnext":
            oid = datatype_config["oid"]
            master_response = await client.getnext(oid=oid)
            response = {master_response.oid: master_response.value}
        elif method == "walk":
            oid = datatype_config["oid"]
            response = {}
            async for binded_var in client.walk(oid=oid):
                response[binded_var.oid] = binded_var.value
        elif method == "multiwalk":
            oids = datatype_config["oid"]
            oids = oids if isinstance(oids, list) else list(oids)
            response = {}
            async for binded_var in client.multiwalk(oids=oids):
                response[binded_var.oid] = binded_var.value
        elif method == "set":
            oid = datatype_config["oid"]
            value = datatype_config["value"]
            response = await client.set(oid=oid, value=value)
        elif method == "multiset":
            mappings = datatype_config["mappings"]
            response = await client.multiset(mappings=mappings)
        elif method == "bulkget":
            scalar_oids = datatype_config.get("scalarOid", [])
            scalar_oids = scalar_oids if isinstance(scalar_oids, list) else list(scalar_oids)
            repeating_oids = datatype_config.get("repeatingOid", [])
            repeating_oids = repeating_oids if isinstance(repeating_oids, list) else list(repeating_oids)
            max_list_size = datatype_config.get("maxListSize", 1)
            response = await client.bulkget(scalar_oids=scalar_oids, repeating_oids=repeating_oids,
                                            max_list_size=max_list_size)
            response = response.scalars
        elif method == "bulkwalk":
            oids = datatype_config["oid"]
            oids = oids if isinstance(oids, list) else list(oids)
            bulk_size = datatype_config.get("bulkSize", 10)
            response = {}
            async for binded_var in client.bulkwalk(bulk_size=bulk_size, oids=oids):
                response[binded_var.oid] = binded_var.value
        elif method == "table":
            oid = datatype_config["oid"]
            num_base_nodes = datatype_config.get("numBaseNodes", 0)
            response = await client.table(oid=oid)
        elif method == "bulktable":
            oid = datatype_config["oid"]
            num_base_nodes = datatype_config.get("numBaseNodes", 0)
            bulk_size = datatype_config.get("bulkSize", 10)
            response = await client.bulktable(oid=oid, bulk_size=bulk_size)
        else:
            self._log.error("Method \"%s\" - Not found", str(method))
        return response

    def __fill_converters(self):
        try:
            for device in self.__devices:
                device["uplink_converter"] = TBModuleLoader.import_module("snmp", device.get('converter',
                                                                                             self._default_converters[
                                                                                                 "uplink"]))(device,
                                                                                                             self._log)
                device["downlink_converter"] = TBModuleLoader.import_module("snmp", device.get('converter',
                                                                                               self._default_converters[
                                                                                                   "downlink"]))(device)
        except Exception as e:
            self._log.exception(e)

    @staticmethod
    def __get_common_parameters(device):
        return {"ip": gethostbyname(device["ip"]),
                "port": device.get("port", 161),
                "timeout": device.get("timeout", 6),
                "community": device["community"],
                }

    def on_attributes_update(self, content):
        try:
            for device in self.__devices:
                if content["device"] == device["deviceName"]:
                    for attribute_request_config in device["attributeUpdateRequests"]:
                        for attribute, value in content["data"]:
                            if search(attribute, attribute_request_config["attributeFilter"]):
                                common_parameters = self.__get_common_parameters(device)
                                result = self.__process_methods(attribute_request_config["method"], common_parameters,
                                                                {**attribute_request_config, "value": value})
                                self._log.debug(
                                    "Received attribute update request for device \"%s\" "
                                    "with attribute \"%s\" and value \"%s\"",
                                    content["device"],
                                    attribute)
                                self._log.debug(result)
                                self._log.debug(content)
        except Exception as e:
            self._log.exception(e)

    def server_side_rpc_handler(self, content):
        try:
            for device in self.__devices:
                if content["device"] == device["deviceName"]:
                    for rpc_request_config in device["serverSideRpcRequests"]:
                        if search(content["data"]["method"], rpc_request_config["requestFilter"]):
                            common_parameters = self.__get_common_parameters(device)
                            result = self.__process_methods(rpc_request_config["method"], common_parameters,
                                                            {**rpc_request_config, "value": content["data"]["params"]})
                            self._log.debug("Received RPC request for device \"%s\" with command \"%s\" and value \"%s\"",
                                      content["device"],
                                      content["data"]["method"])
                            self._log.debug(result)
                            self._log.debug(content)
                            self.__gateway.send_rpc_reply(device=content["device"], req_id=content["data"]["id"],
                                                          content=result)
        except Exception as e:
            self._log.exception(e)
            self.__gateway.send_rpc_reply(device=content["device"], req_id=content["data"]["id"], success_sent=False)



import cmd
import sys
import signal
import multiprocessing.managers
from re import findall
from threading import Thread
from time import sleep
from platform import system as platform_system

from thingsboard_gateway.gateway.shell.proxy import AutoProxy
from thingsboard_gateway.gateway.tb_gateway_service import GatewayManager


class ShellSyntaxError(Exception):
    def __str__(self):
        return 'Invalid syntax'


class Shell(cmd.Cmd, Thread):
    prompt = '(gateway) |> '

    def __init__(self, stdin=None, stdout=None):
        cmd.Cmd.__init__(self, stdin=stdin, stdout=stdout)
        Thread.__init__(self, name='Gateway Shell', daemon=False)

        self.stdout.write('Gateway Shell\n')
        self.stdout.write('=============\n')

        self.gateway_manager, self.gateway = self.create_manager()
        if self.gateway_manager is None or self.gateway is None:
            self.stdout.write('Gateway is not running!\n')
            return

        self.command_config = {
            'storage': [
                {
                    'arg': ('-n', '--name'),
                    'func': self.gateway.get_storage_name
                },
                {
                    'arg': ('-c', '--count'),
                    'func': self.gateway.get_storage_events_count
                }
            ],
            'connector': [
                {
                    'arg': ('-l', '--list'),
                    'func': self.gateway.get_available_connectors
                },
                {
                    'arg': ('-s', '--status'),
                    'val_required': True,
                    'func': self.gateway.get_connector_status
                },
                {
                    'arg': ('-c', '--config'),
                    'val_required': True,
                    'func': self.gateway.get_connector_config
                }
            ],
            'gateway': [
                {
                    'arg': ('-s', '--status'),
                    'func': self.gateway.get_status
                }
            ]
        }

        self.interactive = sys.__stdin__.isatty()

        if hasattr(signal, 'SIGINT'):
            signal.signal(signal.SIGINT, self.interrupt)

        self.start()

    def create_manager(self, timeout=5):
        elapsed_time = 0
        gateway_manager = None
        gateway = None
        self.stdout.write('Establish connection...\n')
        while elapsed_time <= timeout:
            try:
                manager_address = '/tmp/gateway'
                if platform_system() == 'Windows':
                    manager_address = ('127.0.0.1', 9999)
                gateway_manager = GatewayManager(address=manager_address, authkey=b'gateway')
                gateway_manager.connect()
                GatewayManager.register('get_gateway', proxytype=AutoProxy)
                GatewayManager.register('gateway')
                gateway = self.connect(gateway_manager)
            except (FileNotFoundError, AssertionError):
                sleep(1)
                continue
            else:
                break
            finally:
                elapsed_time += 1

        return gateway_manager, gateway

    def connect(self, manager, timeout=5):
        elapsed_time = 0
        gateway = None
        self.connected = False
        self.stdout.write('Connecting to Gateway')
        while not self.connected and elapsed_time <= timeout:
            try:
                gateway = manager.get_gateway()
                gateway.ping()
                self.connected = True
            except (multiprocessing.managers.RemoteError, KeyError, AttributeError, BrokenPipeError):
                self.stdout.write('.')
                gateway = manager.get_gateway()
                self.connected = True
                sleep(1)
            except FileNotFoundError:
                self.stdout.write('.')
                sleep(1)
                continue
            finally:
                elapsed_time += 1

        if not self.connected:
            raise RuntimeError('Timeout error')

        self.stdout.write('\n')
        return gateway

    def run(self):
        self.cmdloop()

    @staticmethod
    def parser(arg, args):
        input_args = findall(r'(-{1,2}[A-Za-z]+)[?^\s]?([A-Za-z\d\s]*)', arg.strip())

        for input_arg in input_args:
            for arg in args:
                opt, val = None, None
                try:
                    (opt, val) = input_arg
                except ValueError:
                    opt = input_arg[0]

                if opt in arg['arg']:
                    if arg.get('val_required') and not val:
                        continue

                    return arg['func'](val.strip()) if val else arg['func']()

        raise ShellSyntaxError()

    def _print(self, *args):
        for arg in args:
            if isinstance(arg, list):
                self.stdout.write(','.join(arg) + '\n')
            elif isinstance(arg, dict):
                for key, value in arg.items():
                    self.stdout.write(str(key).capitalize() + ': ' + str(value) + '\n')
            else:
                self.stdout.write(str(arg) + '\n')

    def wrapper(self, arg, label, config):
        try:
            self._print(self.parser(arg, config))
        except ShellSyntaxError as e:
            self.stdout.write(e.__str__() + ' (see `help ' + label + '`)\n')
            self.do_help(label)
        except (FileNotFoundError, AttributeError, BrokenPipeError, multiprocessing.managers.RemoteError):
            self.stdout.write('Connection is lost...\n')
            self.gateway_manager, self.gateway = self.create_manager()

    # ---
    def do_gateway(self, arg):
        """Gateway
        -s/--status: get gateway status
        """
        self.wrapper(arg, 'gateway', self.command_config['gateway'])

    def do_storage(self, arg):
        """Storage
        -n/--name:   name of storage
        -c/--count:  events in storage
        """
        self.wrapper(arg, 'storage', self.command_config['storage'])

    def do_connector(self, arg):
        """Connector
        -l/--list:             list of available connectors
        -s/--status <name>:    get connector status
        -c/--config <name>:    get connector config
        """
        self.wrapper(arg, 'connector', self.command_config['connector'])

    # ---
    @staticmethod
    def interrupt(_, __):
        sys.stderr.write("\nKeyboard interrupt trapped - use Ctrl+d or Cmd+d to end\n")

    def precmd(self, line: str) -> str:
        return line.strip()

    def postcmd(self, stop: bool, line: str) -> bool:
        return stop

    def emptyline(self) -> bool:
        pass

    @staticmethod
    def do_exit(_):
        """Exit"""
        return -1

    def do_EOF(self, args):
        """EOF"""
        return self.do_exit(args)


def main():
    Shell()


if __name__ == '__main__':
    shell = Shell()



#     Copyright 2022. ThingsBoard
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.

from re import search

from simplejson import dumps

from thingsboard_gateway.gateway.constants import SEND_ON_CHANGE_PARAMETER
from thingsboard_gateway.connectors.mqtt.mqtt_uplink_converter import MqttUplinkConverter
from thingsboard_gateway.tb_utility.tb_utility import TBUtility
from thingsboard_gateway.gateway.statistics_service import StatisticsService


class JsonMqttUplinkConverter(MqttUplinkConverter):
    def __init__(self, config, logger):
        self._log = logger
        self.__config = config.get('converter')
        self.__send_data_on_change = self.__config.get(SEND_ON_CHANGE_PARAMETER)

    @property
    def config(self):
        return self.__config

    @config.setter
    def config(self, value):
        self.__config = value

    @StatisticsService.CollectStatistics(start_stat_type='receivedBytesFromDevices',
                                         end_stat_type='convertedBytesFromDevice')
    def convert(self, topic, data):
        if isinstance(data, list):
            converted_data = []
            for item in data:
                converted_data.append(self._convert_single_item(topic, item))
            self._log.debug(converted_data)
            return converted_data
        else:
            return self._convert_single_item(topic, data)

    def _convert_single_item(self, topic, data):
        datatypes = {"attributes": "attributes",
                     "timeseries": "telemetry"}
        dict_result = {
            "deviceName": self.parse_device_name(topic, data, self.__config),
            "deviceType": self.parse_device_type(topic, data, self.__config),
            "attributes": [],
            "telemetry": []
        }

        if isinstance(self.__send_data_on_change, bool):
            dict_result[SEND_ON_CHANGE_PARAMETER] = self.__send_data_on_change

        try:
            for datatype in datatypes:
                timestamp = data.get("ts", data.get("timestamp")) if datatype == 'timeseries' else None
                dict_result[datatypes[datatype]] = []
                for datatype_config in self.__config.get(datatype, []):
                    if isinstance(datatype_config, str) and datatype_config == "*":
                        for item in data:
                            dict_result[datatypes[datatype]].append(
                                self.create_timeseries_record(item, data[item], timestamp))
                    else:
                        values = TBUtility.get_values(datatype_config["value"], data, datatype_config["type"],
                                                      expression_instead_none=False)
                        values_tags = TBUtility.get_values(datatype_config["value"], data, datatype_config["type"],
                                                           get_tag=True)

                        keys = TBUtility.get_values(datatype_config["key"], data, datatype_config["type"],
                                                    expression_instead_none=False)
                        keys_tags = TBUtility.get_values(datatype_config["key"], data, get_tag=True)

                        full_key = datatype_config["key"]
                        for (key, key_tag) in zip(keys, keys_tags):
                            is_valid_key = "${" in datatype_config["key"] and "}" in \
                                           datatype_config["key"]
                            full_key = full_key.replace('${' + str(key_tag) + '}',
                                                        str(key)) if is_valid_key else key_tag

                        full_value = datatype_config["value"]
                        for (value, value_tag) in zip(values, values_tags):
                            is_valid_value = "${" in datatype_config["value"] and "}" in \
                                             datatype_config["value"]

                            full_value = full_value.replace('${' + str(value_tag) + '}',
                                                            str(value)) if is_valid_value else value

                        if full_key != 'None' and full_value != 'None':
                            dict_result[datatypes[datatype]].append(
                                self.create_timeseries_record(full_key, full_value, timestamp))
        except Exception as e:
            self._log.error('Error in converter, for config: \n%s\n and message: \n%s\n %s', dumps(self.__config),
                            str(data), e)
        self._log.debug(dict_result)
        return dict_result

    @staticmethod
    def create_timeseries_record(key, value, timestamp):
        value_item = {key: value}
        return {"ts": timestamp, 'values': value_item} if timestamp else value_item

    def parse_device_name(self, topic, data, config):
        return self.parse_device_info(
            topic, data, config, "deviceNameJsonExpression", "deviceNameTopicExpression")

    def parse_device_type(self, topic, data, config):
        return self.parse_device_info(
            topic, data, config, "deviceTypeJsonExpression", "deviceTypeTopicExpression")

    def parse_device_info(self, topic, data, config, json_expression_config_name, topic_expression_config_name):
        result = None
        try:
            if config.get(json_expression_config_name) is not None:
                expression = config.get(json_expression_config_name)
                result_tags = TBUtility.get_values(expression, data, get_tag=True)
                result_values = TBUtility.get_values(expression, data, expression_instead_none=True)

                result = expression
                for (result_tag, result_value) in zip(result_tags, result_values):
                    is_valid_key = "${" in expression and "}" in expression
                    result = result.replace('${' + str(result_tag) + '}',
                                            str(result_value)) if is_valid_key else result_tag
            elif config.get(topic_expression_config_name) is not None:
                expression = config.get(topic_expression_config_name)
                search_result = search(expression, topic)
                if search_result is not None:
                    result = search_result.group(0)
                else:
                    self._log.debug(
                        "Regular expression result is None. deviceNameTopicExpression parameter will be interpreted "
                        "as a deviceName\n Topic: %s\nRegex: %s", topic, expression)
                    result = expression
            else:
                self._log.error("The expression for looking \"deviceName\" not found in config %s", dumps(config))
        except Exception as e:
            self._log.error('Error in converter, for config: \n%s\n and message: \n%s\n %s', dumps(config), data, e)
        return result



#     Copyright 2022. ThingsBoard
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.

from os.path import exists
from time import time, sleep
from logging import getLogger
from threading import Thread
from queue import Queue
import datetime

from thingsboard_gateway.storage.sqlite.database_connector import DatabaseConnector
from thingsboard_gateway.storage.sqlite.database_action_type import DatabaseActionType
from thingsboard_gateway.storage.sqlite.storage_settings import StorageSettings

log = getLogger("database")


class Database(Thread):
    """
        What this component does:
        - abstracts creating tables for devices.
        - writes to database
        - reads from database
        - delete data older than specified in config
        ------------- ALL OF THIS IN AN ATOMIC WAY ---------
    """

    def __init__(self, config, processing_queue: Queue):
        super().__init__()
        self.setDaemon(True)
        self.settings = StorageSettings(config)

        if not exists(self.settings.data_folder_path):
            with open(self.settings.data_folder_path, 'w'):
                pass

        # Pass settings to connector
        self.db = DatabaseConnector(self.settings)

        self.db.connect()

        # process Queue
        self.processQueue = processing_queue

        self.__stopped = False

        self.__last_msg_check = time()

        self.msg_counter = 0
        self.start()

    def init_table(self):
        try:
            self.db.execute('''CREATE TABLE IF NOT EXISTS messages (timestamp INTEGER, message TEXT); ''')
            self.db.commit()
        except Exception as e:
            self.db.rollback()
            log.exception(e)

    def run(self):
        while True:
            self.process()

            sleep(.2)

    def process(self):
        try:
            if time() - self.__last_msg_check >= self.settings.messages_ttl_check_in_hours:
                self.__last_msg_check = time()
                self.delete_data_lte(self.settings.messages_ttl_in_days)

            # Signalization so that we can spam call process()
            if not self.__stopped and self.processQueue:
                while self.processQueue.qsize() > 0:

                    req = self.processQueue.get()

                    log.debug("Processing %s" % req.type)
                    if req.type is DatabaseActionType.WRITE_DATA_STORAGE:

                        message = req.data

                        timestamp = time()

                        self.db.execute('''INSERT INTO messages (timestamp, message) VALUES (?, ?);''',
                                        [timestamp, message])

                        self.db.commit()
            else:
                log.error("Storage is closed!")

        except Exception as e:
            self.db.rollback()
            log.exception(e)

    def read_data(self):
        try:
            data = self.db.execute('''SELECT timestamp, message FROM messages ORDER BY timestamp ASC LIMIT 0, 50;''')
            return data
        except Exception as e:
            self.db.rollback()
            log.exception(e)

    def delete_data(self, ts):
        try:
            data = self.db.execute('''DELETE FROM messages WHERE timestamp <= ?;''', [ts,])
            return data
        except Exception as e:
            self.db.rollback()
            log.exception(e)

    def delete_data_lte(self, days):
        try:
            ts = (datetime.datetime.now() - datetime.timedelta(days=days)).timestamp()
            data = self.db.execute('''DELETE FROM messages WHERE timestamp <= ? ;''', [ts])
            return data
        except Exception as e:
            self.db.rollback()
            log.exception(e)

    def setProcessQueue(self, process_queue):
        self.processQueue = process_queue

    def closeDB(self):
        self.db.close()



#     Copyright 2022. ThingsBoard
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.

import asyncio
import re
from random import choice
from string import ascii_lowercase
from threading import Thread
from time import sleep, time
from queue import Queue

from thingsboard_gateway.connectors.connector import Connector
from thingsboard_gateway.connectors.opcua_asyncio.device import Device
from thingsboard_gateway.tb_utility.tb_loader import TBModuleLoader
from thingsboard_gateway.tb_utility.tb_utility import TBUtility
from thingsboard_gateway.tb_utility.tb_logger import init_logger

try:
    import asyncua
except ImportError:
    print("OPC-UA library not found")
    TBUtility.install_package("asyncua")
    import asyncua

from asyncua.crypto.security_policies import SecurityPolicyBasic256Sha256, SecurityPolicyBasic256, \
    SecurityPolicyBasic128Rsa15
from asyncua.ua.uaerrors import UaStatusCodeError, BadNodeIdUnknown, BadConnectionClosed, \
    BadInvalidState, BadSessionClosed, BadAttributeIdInvalid, BadCommunicationError, BadOutOfService

DEFAULT_UPLINK_CONVERTER = 'OpcUaUplinkConverter'

SECURITY_POLICIES = {
    "Basic128Rsa15": SecurityPolicyBasic128Rsa15,
    "Basic256": SecurityPolicyBasic256,
    "Basic256Sha256": SecurityPolicyBasic256Sha256,
}


class OpcUaConnectorAsyncIO(Connector, Thread):
    def __init__(self, gateway, config, connector_type):
        self.statistics = {'MessagesReceived': 0,
                           'MessagesSent': 0}
        super().__init__()
        self._connector_type = connector_type
        self.__gateway = gateway
        self.__config = config
        self.__server_conf = config['server']
        self.setName(
            self.__config.get("name", 'OPC-UA Connector ' + ''.join(choice(ascii_lowercase) for _ in range(5))))
        self.__log = init_logger(self.__gateway, self.name, self.__config.get('logLevel', 'INFO'))

        if "opc.tcp" not in self.__server_conf.get("url"):
            self.__opcua_url = "opc.tcp://" + self.__server_conf.get("url")
        else:
            self.__opcua_url = self.__server_conf.get("url")

        self.__data_to_send = Queue(-1)
        self.__sub_data_to_convert = Queue(-1)

        self.__loop = asyncio.new_event_loop()

        self.__client = None
        self.__subscription = None

        self.__connected = False
        self.__stopped = False
        self.daemon = True

        self.__device_nodes = []
        self.__last_poll = 0

    def open(self):
        self.__stopped = False
        self.start()
        self.__log.info("Starting OPC-UA Connector (Async IO)")

    def get_type(self):
        return self._connector_type

    def close(self):
        task = self.__loop.create_task(self.__reset_nodes())

        while not task.done():
            sleep(.2)

        self.__stopped = True
        self.__connected = False
        self.__log.info('%s has been stopped.', self.get_name())
        self.__log.reset()

    async def __reset_node(self, node):
        node['valid'] = False
        if node.get('sub_on', False):
            try:
                if self.__subscription:
                    await self.__subscription.unsubscribe(node['subscription'])
            except:
                pass
            node['subscription'] = None
            node['sub_on'] = False

    async def __reset_nodes(self, device_name=None):
        for device in self.__device_nodes:
            if device_name is None or device.name == device_name:
                for section in ('attributes', 'timeseries'):
                    for node in device.values.get(section, []):
                        await self.__reset_node(node)

        if device_name is None and self.__subscription:
            try:
                await self.__subscription.delete()
            except:
                pass
            self.__subscription = None

    def get_name(self):
        return self.name

    def is_connected(self):
        return self.__connected

    def get_config(self):
        return self.__server_conf

    def run(self):
        data_send_thread = Thread(name='Send Data Thread', target=self.__send_data, daemon=True)
        data_send_thread.start()

        if not self.__server_conf.get('disableSubscriptions', False):
            sub_data_convert_thread = Thread(name='Sub Data Convert Thread', target=self.__convert_sub_data,
                                             daemon=True)
            sub_data_convert_thread.start()

        self.__loop.run_until_complete(self.start_client())

    async def start_client(self):
        while not self.__stopped:
            self.__client = asyncua.Client(url=self.__opcua_url,
                                           timeout=self.__server_conf.get('timeoutInMillis', 4000) / 1000)

            if self.__server_conf["identity"].get("type") == "cert.PEM":
                await self.__set_auth_settings_by_cert()
            if self.__server_conf["identity"].get("username"):
                self.__set_auth_settings_by_username()

            try:
                async with self.__client:
                    self.__connected = True

                    try:
                        await self.__client.load_data_type_definitions()
                    except Exception as e:
                        self.__log.error("Error on loading type definitions:\n %s", e)

                    while not self.__stopped:
                        if time() - self.__last_poll >= self.__server_conf.get('scanPeriodInMillis', 5000) / 1000:
                            await self.__scan_device_nodes()
                            await self.__poll_nodes()
                            self.__last_poll = time()

                        await asyncio.sleep(.2)
            except (ConnectionError, BadSessionClosed):
                self.__log.warning('Connection lost for %s', self.get_name())
            except asyncio.exceptions.TimeoutError:
                self.__log.warning('Failed to connect %s', self.get_name())
            except Exception as e:
                self.__log.exception(e)
            finally:
                await self.__reset_nodes()
                self.__connected = False

    async def __set_auth_settings_by_cert(self):
        try:
            ca_cert = self.__server_conf["identity"].get("caCert")
            private_key = self.__server_conf["identity"].get("privateKey")
            cert = self.__server_conf["identity"].get("cert")
            policy = self.__server_conf["security"]

            if cert is None or private_key is None:
                self.__log.exception("Error in ssl configuration - cert or privateKey parameter not found")
                raise RuntimeError("Error in ssl configuration - cert or privateKey parameter not found")

            await self.__client.set_security(
                SECURITY_POLICIES[policy],
                certificate=cert,
                private_key=private_key,
                server_certificate=ca_cert
            )
        except Exception as e:
            self.__log.exception(e)

    def __set_auth_settings_by_username(self):
        self.__client.set_user(self.__server_conf["identity"].get("username"))
        if self.__server_conf["identity"].get("password"):
            self.__client.set_password(self.__server_conf["identity"].get("password"))

    def __load_converter(self, device):
        converter_class_name = device.get('converter', DEFAULT_UPLINK_CONVERTER)
        module = TBModuleLoader.import_module(self._connector_type, converter_class_name)

        if module:
            self.__log.debug('Converter %s for device %s - found!', converter_class_name, self.name)
            return module

        self.__log.error("Cannot find converter for %s device", self.name)
        return None

    @staticmethod
    def is_regex_pattern(pattern):
        return not re.fullmatch(pattern, pattern)

    async def __find_nodes(self, node_list, current_parent_node, nodes):
        assert len(node_list) > 0
        final = []

        children = await current_parent_node.get_children()
        for node in children:
            child_node = await node.read_browse_name()

            if re.fullmatch(node_list[0], child_node.Name):
                new_nodes = [*nodes, f'{child_node.NamespaceIndex}:{child_node.Name}']
                if len(node_list) == 1:
                    final.append(new_nodes)
                else:
                    final.extend(await self.__find_nodes(node_list[1:], current_parent_node=node, nodes=new_nodes))

        return final

    async def find_nodes(self, node_pattern, current_parent_node=None, nodes=[]):
        node_list = node_pattern.split('\\.')

        if current_parent_node is None:
            current_parent_node = self.__client.nodes.root

            if len(node_list) > 0 and node_list[0].lower() == 'root':
                node_list = node_list[1:]

        return await self.__find_nodes(node_list, current_parent_node, nodes)

    async def find_node_name_space_index(self, path):
        if isinstance(path, str):
            path = path.split('\\.')

        # find unresolved nodes
        u_node_count = len(tuple(filter(lambda u_node: len(u_node.split(':')) < 2, path)))

        resolved = path[:-u_node_count]
        resolved_level = len(path) - u_node_count
        parent_node = await self.__client.nodes.root.get_child(resolved)

        unresolved = path[resolved_level:]
        return await self.__find_nodes(unresolved, current_parent_node=parent_node, nodes=resolved)

    async def __scan_device_nodes(self):
        existing_devices = list(map(lambda dev: dev.name, self.__device_nodes))

        scanned_devices = []
        for device in self.__server_conf.get('mapping', []):
            nodes = await self.find_nodes(device['deviceNodePattern'])
            self.__log.debug('Found devices: %s', nodes)

            device_names = []
            device_name_pattern = device['deviceNamePattern']
            if re.match(r"\${([A-Za-z.:\d]*)}", device_name_pattern):
                device_name_nodes = await self.find_nodes(device_name_pattern)
                self.__log.debug('Found device name nodes: %s', device_name_nodes)

                for node in device_name_nodes:
                    try:
                        var = await self.__client.nodes.root.get_child(node)
                        value = await var.read_value()
                        device_names.append(value)
                    except Exception as e:
                        self.__log.exception(e)
                        continue
            else:
                device_names.append(device_name_pattern)

            for device_name in device_names:
                scanned_devices.append(device_name)
                if device_name not in existing_devices:
                    for node in nodes:
                        converter = self.__load_converter(device)
                        device_config = {**device, 'device_name': device_name}
                        self.__device_nodes.append(
                            Device(path=node, name=device_name, config=device_config,
                                   converter=converter(device_config, self.__log),
                                   converter_for_sub=converter(device_config, self.__log) if not self.__server_conf.get(
                                       'disableSubscriptions',
                                       False) else None, logger=self.__log))
                        self.__log.info('Added device node: %s', device_name)

        for device_name in existing_devices:
            if device_name not in scanned_devices:
                await self.__reset_nodes(device_name)

        self.__log.debug('Device nodes: %s', self.__device_nodes)

    def __convert_sub_data(self):
        while not self.__stopped:
            if not self.__sub_data_to_convert.empty():
                sub_node, data = self.__sub_data_to_convert.get()

                for device in self.__device_nodes:
                    for section in ('attributes', 'timeseries'):
                        for node in device.values.get(section, []):
                            if node.get('id') == sub_node.__str__():
                                device.converter_for_sub.convert(config={'section': section, 'key': node['key']},
                                                                 val=data.monitored_item.Value)
                                converter_data = device.converter_for_sub.get_data()

                                if converter_data:
                                    self.__data_to_send.put(*converter_data)
                                    device.converter_for_sub.clear_data()
            else:
                sleep(.2)

    async def __poll_nodes(self):
        for device in self.__device_nodes:
            for section in ('attributes', 'timeseries'):
                for node in device.values.get(section, []):
                    try:
                        path = node.get('qualified_path', node['path'])
                        if isinstance(path, str) and re.match(r"(ns=\d+;[isgb]=[^}]+)", path):
                            var = self.__client.get_node(path)
                        else:
                            if len(path[-1].split(':')) != 2:
                                qualified_path = await self.find_node_name_space_index(path)
                                if len(qualified_path) == 0:
                                    if node.get('valid', True):
                                        self.__log.warning('Node not found; device: %s, key: %s, path: %s', device.name,
                                                           node['key'], node['path'])
                                        await self.__reset_node(node)
                                    continue
                                elif len(qualified_path) > 1:
                                    self.__log.warning(
                                        'Multiple matching nodes found; device: %s, key: %s, path: %s; %s', device.name,
                                        node['key'], node['path'], qualified_path)
                                node['qualified_path'] = qualified_path[0]
                                path = qualified_path[0]

                            var = await self.__client.nodes.root.get_child(path)

                        if not node.get('valid', False) or self.__server_conf.get('disableSubscriptions', False):
                            value = await var.read_data_value()
                            device.converter.convert(config={'section': section, 'key': node['key']}, val=value)

                            if not self.__server_conf.get('disableSubscriptions', False) and not node.get('sub_on',
                                                                                                          False):
                                if self.__subscription is None:
                                    self.__subscription = await self.__client.create_subscription(1, SubHandler(
                                        self.__sub_data_to_convert, self.__log))
                                handle = await self.__subscription.subscribe_data_change(var)
                                node['subscription'] = handle
                                node['sub_on'] = True
                                node['id'] = var.nodeid.to_string()
                                self.__log.info("Subscribed on data change; device: %s, key: %s, path: %s", device.name,
                                                node['key'], node['id'])

                            node['valid'] = True
                    except ConnectionError:
                        raise
                    except (BadNodeIdUnknown, BadConnectionClosed, BadInvalidState, BadAttributeIdInvalid,
                            BadCommunicationError, BadOutOfService):
                        if node.get('valid', True):
                            self.__log.warning('Node not found (2); device: %s, key: %s, path: %s', device.name,
                                               node['key'], node['path'])
                            await self.__reset_node(node)
                    except UaStatusCodeError as uae:
                        if node.get('valid', True):
                            self.__log.exception('Node status code error: %s', uae)
                            await self.__reset_node(node)
                    except Exception as e:
                        if node.get('valid', True):
                            self.__log.exception(e)
                            await self.__reset_node(node)

            converter_data = device.converter.get_data()
            if converter_data:
                self.__data_to_send.put(*converter_data)

                device.converter.clear_data()

    def __send_data(self):
        while not self.__stopped:
            if not self.__data_to_send.empty():
                data = self.__data_to_send.get()
                self.statistics['MessagesReceived'] = self.statistics['MessagesReceived'] + 1
                self.__log.debug(data)
                self.__gateway.send_to_storage(self.get_name(), data)
                self.statistics['MessagesSent'] = self.statistics['MessagesSent'] + 1
                self.__log.debug('Data to ThingsBoard %s', data)
            else:
                sleep(.2)

    async def get_shared_attr_node_id(self, path, result={}):
        try:
            q_path = await self.find_node_name_space_index(path)
            var = await self.__client.nodes.root.get_child(q_path[0])
            result['result'] = var
        except Exception as e:
            result['error'] = e.__str__()

    def on_attributes_update(self, content):
        self.__log.debug(content)
        try:
            device = tuple(filter(lambda i: i.name == content['device'], self.__device_nodes))[0]

            for (key, value) in content['data'].items():
                for attr_update in device.config['attributes_updates']:
                    if attr_update['attributeOnThingsBoard'] == key:
                        result = {}
                        task = self.__loop.create_task(
                            self.get_shared_attr_node_id(
                                device.path + attr_update['attributeOnDevice'].replace('\\', '').split('.'), result))

                        while not task.done():
                            sleep(.1)

                        if result.get('error'):
                            self.__log.error('Node not found! (%s)', result['error'])
                            return

                        node_id = result['result']
                        self.__loop.create_task(self.__write_value(node_id, value))
                        return
        except Exception as e:
            self.__log.exception(e)

    def server_side_rpc_handler(self, content):
        try:
            if content.get('data') is None:
                content['data'] = {'params': content['params'], 'method': content['method']}

            rpc_method = content["data"].get("method")

            # check if RPC type is connector RPC (can be only 'get' or 'set')
            try:
                (connector_type, rpc_method_name) = rpc_method.split('_')
                if connector_type == self._connector_type:
                    rpc_method = rpc_method_name
                    content['device'] = content['params'].split(' ')[0].split('=')[-1]
            except (ValueError, IndexError):
                pass

            # firstly check if a method is not service
            if rpc_method == 'set' or rpc_method == 'get':
                full_path = ''
                args_list = []
                device = content.get('device')

                try:
                    args_list = content['data']['params'].split(';')

                    if 'ns' in content['data']['params']:
                        full_path = ';'.join([item for item in (args_list[0:-1] if rpc_method == 'set' else args_list)])
                    else:
                        full_path = args_list[0].split('=')[-1]
                except IndexError:
                    self.__log.error('Not enough arguments. Expected min 2.')
                    self.__gateway.send_rpc_reply(device=device,
                                                  req_id=content['data'].get('id'),
                                                  content={content['data'][
                                                               'method']: 'Not enough arguments. Expected min 2.',
                                                           'code': 400})

                result = {}
                if rpc_method == 'get':
                    task = self.__loop.create_task(self.__read_value(full_path, result))

                    while not task.done():
                        sleep(.2)
                elif rpc_method == 'set':
                    value = args_list[2].split('=')[-1]
                    task = self.__loop.create_task(self.__write_value(full_path, value, result))

                    while not task.done():
                        sleep(.2)

                self.__gateway.send_rpc_reply(device=device,
                                              req_id=content['data'].get('id'),
                                              content={content['data']['method']: result})
            else:
                device = tuple(filter(lambda i: i.name == content['device'], self.__device_nodes))[0]

                for rpc in device.config['rpc_methods']:
                    if rpc['method'] == content["data"]['method']:
                        arguments_from_config = rpc["arguments"]
                        arguments = content["data"].get("params") if content["data"].get(
                            "params") is not None else arguments_from_config

                        try:
                            result = {}
                            task = self.__loop.create_task(self.__call_method(device.path, arguments, result))

                            while not task.done():
                                sleep(.2)

                            self.__gateway.send_rpc_reply(content["device"],
                                                          content["data"]["id"],
                                                          {content["data"]["method"]: result, "code": 200})
                            self.__log.debug("method %s result is: %s", rpc['method'], result)
                        except Exception as e:
                            self.__log.exception(e)
                            self.__gateway.send_rpc_reply(content["device"], content["data"]["id"],
                                                          {"error": str(e), "code": 500})
                    else:
                        self.__log.error("Method %s not found for device %s", rpc_method, content["device"])
                        self.__gateway.send_rpc_reply(content["device"], content["data"]["id"],
                                                      {"error": "%s - Method not found" % rpc_method,
                                                       "code": 404})

        except Exception as e:
            self.__log.exception(e)

    async def __write_value(self, path, value, result={}):
        try:
            var = path
            if isinstance(path, str):
                var = self.__client.get_node(path.replace('\\.', '.'))

            await var.write_value(value)
        except Exception as e:
            result['error'] = e.__str__()

    async def __read_value(self, path, result={}):
        try:
            var = self.__client.get_node(path)
            result['value'] = await var.read_value()
        except Exception as e:
            result['error'] = e.__str__()

    async def __call_method(self, path, arguments, result={}):
        try:
            var = self.__client.get_node(path)
            result['result'] = await var.call_method(*arguments)
        except Exception as e:
            result['error'] = e.__str__()

    def update_converter_config(self, converter_name, config):
        self.__log.debug('Received remote converter configuration update for %s with configuration %s', converter_name,
                         config)
        for device in self.__device_nodes:
            if device.converter.__class__.__name__ == converter_name:
                device.config.update(config)
                device.load_values()
                self.__log.info('Updated converter configuration for: %s with configuration %s',
                                converter_name, device.config)

                for node_config in self.__server_conf['mapping']:
                    if node_config['deviceNodePattern'] == device.config['deviceNodePattern']:
                        node_config.update(config)

                self.__gateway.update_connector_config_file(self.name, {'server': self.__server_conf})


class SubHandler:
    def __init__(self, queue, logger):
        self.__log = logger
        self.__queue = queue

    def datachange_notification(self, node, _, data):
        self.__log.debug("New data change event %s %s", node, data)
        self.__queue.put((node, data))
